<?php

namespace Drupal\tmgmt_wordsonline\Plugin\tmgmt\Translator;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\tmgmt\ContinuousTranslatorInterface;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\Entity\RemoteMapping;
use Drupal\tmgmt\JobItemInterface;
use Drupal\tmgmt\RemoteMappingInterface;
use Drupal\tmgmt\SourcePreviewInterface;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt\TranslatorPluginBase;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\ClientInterface;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\Translator\AvailableResult;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Form\FormBuilder;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\tmgmt_wordsonline\WordsOnlineConst;
use Drupal\tmgmt_wordsonline\WordsOnlineMessage;
use Drupal\tmgmt_wordsonline\Entity\WOLLangPair;
use Drupal\tmgmt_wordsonline\Entity\WOLServiceLevel;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileSystem;
use Drupal\tmgmt_wordsonline\Entity\WOFile;
use Drupal\tmgmt\Translator\TranslatableResult;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7;
/**
 * WordsOnline translation plugin controller.
 *
 * @TranslatorPlugin(
 *   id = "wordsonline",
 *   label = @Translation("WordsOnline"),
 *   description = @Translation("WordsOnline translator service."),
 *   ui = "Drupal\tmgmt_wordsonline\WordsOnlineTranslatorUi",
 *   logo = "icons/wordsonline-startup.png",
 * )
 */
class WordsOnlineTranslator extends TranslatorPluginBase implements ContainerFactoryPluginInterface, ContinuousTranslatorInterface {
  /**
   * The translator.
   *
   * @var TranslatorInterface
   */
  private $translator;


  /**
   * Guzzle HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $client;
 /**
   * Constructs a WordsOnlineTranslator object.
   *
   * @param \GuzzleHttp\ClientInterface $client
   *   The Guzzle HTTP client.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param array $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\tmgmt_file\Format\FormatManager $format_manager
   *   The TMGMT file format manager.
   */
  public function __construct(ClientInterface $client, array $configuration, $plugin_id, array $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = $client;
    $this->setConfiguration($configuration);
  }
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    /** @var \GuzzleHttp\ClientInterface $client */
    $client = $container->get('http_client');
    return new static(
      $client,
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }
 /**
   * {@inheritdoc}
   */
  public function getSupportedRemoteLanguages(TranslatorInterface $translator) {
    $supported_remote_languages = [];
    $database = \Drupal::database();
    $results =  $database->select('tmgmt_wordsonline_support_languges', 'wol')
                         ->fields('wol')
                         ->execute()
                         ->fetchAll();
    if($results != null && count($results)>0){
      foreach ($results as $lang ) {
        $code =$lang->code;
        $lang_name=$lang->lang_name;
        if($code!="")
         $supported_remote_languages[$code] = $lang_name;
      }
    }
    return $supported_remote_languages;
  }
  /**
   * Sets a Translator.
   *
   * @param \Drupal\tmgmt\TranslatorInterface $translator
   *   The translator to set.
   */
  public function setTranslator(TranslatorInterface $translator) {
    $this->translator = $translator;
  }
   /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'token' =>''
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = NestedArray::mergeDeep($this->defaultConfiguration(), $configuration);
  }
  /**
   * Does a request to wordsonline services.
   *
   * @param string $path
   *   Resource path.
   * @param string $method
   *   (Optional) HTTP method (GET, POST...). By default uses GET method.
   * @param array $params
   *   (Optional) Form parameters to send to wordsonline service.
   * @param bool $download
   *   (Optional) If we expect resource to be downloaded. FALSE by default.
   * @param bool $code
   *   (Optional) If we want to return the status code of the call. FALSE by
   *   default.
   *
   * @return array
   *   Response array from wordsonline.
   *
   * @throws \Drupal\tmgmt\TMGMTException
   */
  public function request($path, $method = 'GET', $params = [], $download = FALSE, $code = FALSE) {
    $options = [];
    if (!$this->translator) {
      throw new TMGMTException('There is no Translator entity. Access to the client contact key is not possible.');
    }

    $url = WordsOnlineConst::API_URL  . $path;  
    try {
      if ($method == 'GET') {
        $options['query'] = $params;
      }
      else {
        $options['json'] = $params;
      }
      $options['headers'] = [
        'Authorization' => 'Bearer '.$this->getConfiguration()['token'],
        'Content-Type' => 'application/json'
      ];
      $response = $this->client->request($method, $url, $options);
    }
    catch (RequestException $e) {
      if (!$e->hasResponse()) {
        if ($code) {
          return $e->getCode();
        }
       
        throw new TMGMTException(WordsOnlineMessage::UNABLE_TO_CONNECT_SERVICES, ['@error' => $e->getMessage()], $e->getCode());
      }
      $response = $e->getResponse();
     
      if ($code) {
        return $response->getStatusCode();
      }
     
      throw new TMGMTException(WordsOnlineMessage::UNABLE_TO_CONNECT_SERVICES, ['@error' => $response->getReasonPhrase()], $response->getStatusCode());
    }
    $received_data = $response->getBody()->getContents();
    
   
    if ($response->getStatusCode() != 200) {
      throw new TMGMTException(WordsOnlineMessage::UNABLE_TO_CONNECT_SERVICES_WITH_URL,
        ['@error' => $response->getStatusCode(), '@url' => $url]);
    }

    // If we are expecting a download, just return received data.
    if ($download) {
      return $received_data;
    }
   
    $received_data = json_decode($received_data, TRUE);
    if($response->getStatusCode() == 200 && $path =="token"){
      $this->defaultConfiguration()['token']=$received_data['access_token'];
      $this->setConfiguration($this->configuration);
    }
    return $received_data;
  }
  /**
   * Fetches translations for the given job.
   *
   * @param \Drupal\tmgmt\JobInterface $job
   *   A job containing job items that translations will be fetched for.
   */
  public function fetchTranslations(JobInterface $job) {
    
  }
  /**
   * {@inheritdoc}
   */
  public function requestJobItemsTranslation(array $job_items) {

  }

   /**
   * {@inheritdoc}
   */
  public function requestTranslation(JobInterface $job) {
    $translator = $job->getTranslator();
    $translator->setSetting('xliff_processing', TRUE);
    $translator->save();

    try {

      $files=[];
      $source_lang =$job->getSourceLangcode();
      $target_lang = $job->getTargetLangcode();
      $request_name = $job->getSetting('request_name');;
      $projectId = $job->getSetting('project_key');
      if($projectId == null){
        $projectId ='';
      }     

      $results = \Drupal::database()->query('SELECT  code,wol_code FROM tmgmt_wordsonline_support_languges')->fetchAll();
      foreach ($results as $row) {
          if($row->code == $source_lang){
           $source_lang = $row->wol_code;
          }
          if($row->code == $target_lang){
           $target_lang = $row->wol_code;
          }
      }
      $params['request_name'] = $request_name;
      $params['projectId'] = $projectId;
      $params['sourceLanguage']= $source_lang;
      $params['targetLanguages']= $target_lang;
      $params['contentTypeId']=$job->getSetting('content_type');
      $params['serviceLevel']=$job->getSetting('service_level');
      $auto_approve_quote =$job->getSetting('auto_approve_quote');
      $params['auto_approve_quote']=$auto_approve_quote == true || $auto_approve_quote == 1? 'true':'false';
      $conn = \Drupal\Core\Database\Database::getConnection();
      $job_items =array_values($job->getItems());
      
      foreach ($job_items as $job_item) {
        $job_item_id = $job_item->id();
        $xliff = \Drupal::service('plugin.manager.tmgmt_file.format')->createInstance('xlf', ['target' => 'source']);
        $conditions = ['tjiid' => ['value' => $job_item_id]];
        $xliff_content = $xliff->export($job,$conditions);
        // Build a file name.
        $file_name = "Job_{$job->id()}_{$job_item_id}.xlf";
        $path = "public://";
        if (\Drupal::service('file_system')->prepareDirectory($path, \Drupal\Core\File\FileSystemInterface::CREATE_DIRECTORY)) {
          if( (float)(\Drupal::VERSION) >=9.3){
            \Drupal::service('file.repository')->writeData(
             $xliff_content,
             $path .$file_name
            );
          }else{
            /** @var \Drupal\file\FileInterface $file */
            $file = file_save_data(
              $xliff_content,
              $path .$file_name
            );
          }

        }

        $fields["job_id"] = $job->id();
        $fields["job_item_id"] = $job_item_id;
        $fields["file_name"] =  $file_name;
        $fields["file_path"] = $path . $file_name;
        $fields["source_language"] =  $source_lang;
        $fields["target_language"] =  $target_lang;
        $f_ret = \Drupal::database()->query("SELECT  * FROM tmgmt_wordsonline_job_files where job_id = :job_id and job_item_id = :job_item_id LIMIT 1",[':job_id' =>$job->id(),':job_item_id' =>$job_item_id])->fetchAssoc(); 
        if($f_ret != null){
          \Drupal::database()->query("Delete FROM tmgmt_wordsonline_job_files where job_id = :job_id and job_item_id = :job_item_id LIMIT 1",[':job_id' =>$job->id(),':job_item_id' =>$job_item_id])->fetchAssoc(); 
        }
        $conn->insert('tmgmt_wordsonline_job_files')
             ->fields($fields)->execute();
        if ($job_item->getJob()->isContinuous()) {
                $job_item->active();
          }
      }

      $f_ret = \Drupal::database()->query("SELECT  * FROM tmgmt_wordsonline_jobs where job_id = :job_id  and status = :s LIMIT 1",[':job_id' =>$job->id(),':s' =>WordsOnlineConst::JOB_NEW ])->fetchAssoc(); 
      if($f_ret == null){
        $orders["job_id"]=$job->id();
        $orders["file_guid"]='';
        $orders["request_guid"]='';
        $orders["request_name"] = $request_name;
        $orders["project_guid"] = $projectId;
        $orders["status"] = WordsOnlineConst::JOB_NEW;
        $conn->insert(WordsOnlineConst::JOB_TABLE)
             ->fields($orders)->execute();
      }

      $params['job_id']=$job->id();
      $results = \Drupal::database()->query('SELECT  * FROM tmgmt_wordsonline_configs LIMIT 1')->fetchAssoc(); 
      $is_created =false;
      if($results != null){
        $auth["username"] = $results["username"];
        $auth["password"] = $results["password"];
        $auth["scope"] = $results["scope"];
        $auth["grant_type"] =$results["grant_type"];   
        $response = getToken('POST', $auth);
        $response = json_decode($response, TRUE);
        $token =$response['access_token'];
        $is_created = $this->createOrder(WordsOnlineConst::CREATE_REQUEST_URL,'POST',$params, $path .$file_name,$token);
      }else{
        \Drupal::messenger()->addError(t('Cannot find WordsOnline provider settings. Please contact <a href="mailto:helpdesk@wordsonline.com"> helpdesk@wordsonline.com</a> to obtain WordsOnline provider settings.'));
      }
      if($is_created == false){
       \Drupal::service('file_system')->delete($path . $file_name);
        return;
      }
      $job->submitted(t('The translation job has been submitted. Please go to <a href="'.base_path().'admin/tmgmt/wordsonline/jobs'.'"> WordsOnline Translation Request</a> to see your quote.'));
      \Drupal::service('file_system')->delete($path . $file_name);

    }
    catch (TMGMTException $e) {
    
      $job->rejected('Job has been rejected with following error: @error', ['@error' => $e->getMessage()], 'error');
    }
  }
  /**
   * create WordsOnline order
   *
   */
  function createOrder($path, $method = 'POST', $params = [],$file_path,$token = NULL) {
    $options = [];
     $job_id = $params['job_id'];
    $url = WordsOnlineConst::API_URL  . $path;
    $client = new \GuzzleHttp\Client();

    try {
      $options['headers'] = [
        'Authorization' => 'Bearer '.$token,
        'Referer'=>'ClientAPI',
      ];

      $options['multipart'] = [
      [
        'name'      => 'sourceFiles',
        'filename' =>  $file_path,
        'contents' => fopen( $file_path, 'r'),
        'headers'  => [
          'Content-Type' => '<Content-type header>'
        ]
      ],
      [
        'name'      => 'requestName',
        'contents' => $params['request_name'],
      ],  
      [
        'name'      => 'projectId',
        'contents' => $params['projectId'],
      ], 
      [
        'name'      => 'sourceLanguage',
        'contents' => $params['sourceLanguage'],
      ],
      [
        'name'      => 'targetLanguages',
        'contents' => $params['targetLanguages'],
      ], 
      [
        'name'      => 'contentTypeId',
        'contents' => $params['contentTypeId'],
      ], 
      [
        'name'      => 'serviceLevel',
        'contents' =>  $params['serviceLevel'],
      ], 
      [
        'name'      => 'extraProofreading',
        'contents' => 'false',
      ], 
      [
        'name'      => 'dtpService',
        'contents' => 'false',
      ], 
      [
        'name'      => 'isAutoApprove',
        'contents' => $params['auto_approve_quote'] ,
      ],  
      [
        'name'      => 'description',
        'contents' => 'Request from drupal',
      ]
       
      ];
      $options['timeout']= 3600;
      $response = $client->post($url, $options);
   
    }
    catch (RequestException $e) {

      \Drupal::logger(WordsOnlineConst::JOB_TABLE)->error($e->getMessage());
      \Drupal::messenger()->addError(WordsOnlineMessage::FAILED_TO_CREATE_NEW_ORDER);
      return false;
    }
    $received_data = $response->getBody()->getContents();
    $err_msg =WordsOnlineMessage::FAILED_TO_CREATE_NEW_ORDER;
    if ($response->getStatusCode() != 200) {
      \Drupal::messenger()->addError(WordsOnlineMessage::FAILED_TO_CREATE_NEW_ORDER);
      return false;
    }
    $orderResponse = json_decode($received_data, TRUE); 
    if($orderResponse["status"] ==1){
      $conn = \Drupal\Core\Database\Database::getConnection();
      $conn->update(WordsOnlineConst::JOB_TABLE)
           ->condition('job_id', $job_id) 
           ->fields(['status' => WordsOnlineConst::JOB_CREATED,'request_guid'=>$orderResponse["result"]])
           ->execute();
      \Drupal::messenger()->addMessage(WordsOnlineMessage::NEW_ORDER_CREATED);
      return true;
    }else{
      if($orderResponse!=null && $orderResponse['message'] != null ){
        $err_msg =  $err_msg ."\r\n".  $orderResponse['message'];
      }
      \Drupal::messenger()->addError( $err_msg);
    }
    return false;
  }
}